classdef RigidBody
    %object.transform.RigidBody  Rigid body geometric transformation.
    %
    % This transformation maps Euclidean 3D points to Euclidean 3D points,
    % preserving collineation, incidency, lengths, and angles.
    %
    %  @author  enrique.munoz@iit.it
    %
    %  @projectid  OMRON.P37505
    %
    %  @date    2013-06-05
    %
    %  @lastmod 2013-06-05
    %
    %  @changelog 2013-06-05  enrique.munoz@iit.it
    %
    %             Initial version
    %
    properties
        %% Properties
        
        R = object.transform.Rotation(eye(3)) ;
        % 3x3 matrix defining the rotation.
        
        t = zeros(3,1) ;
        % 3x1 column vector defining the translation
        
    end% Properties
    
    properties (Dependent = true)
        
        Rotation
        
        Translation
        
    end
        
    methods
        %% Methods
        
        function obj = RigidBody(R, t)
        %% Constructor for RigidBody class    
        
        if ( nargin == 1 )
        
            obj.R  = object.transform.Rotation(euler2rotmatrix(R(1:3))) ;
            obj.t  = R(4:6) ;
            obj.t  = obj.t(:) ;
        else
            % Check if R is a rotation matrix
            if isa(R, 'object.transform.Rotation')
                
                %
                obj.R = R ;
                
            else
                
                if (~isequal(size(R), [3 3]))
                    error('Rotation must be a 3x3 matrix') ;
                end
                
                % Assign the rotation matrix
                obj.R = object.transform.Rotation(R) ;
                
            end
            
            % Check if t is a translation vector
            if (~isequal(size(t), [3 1]))
                error('Translation must be a 3x1 vector') ;
            end
             
            % Assign the translation vector
            obj.t = t ;
        
        end
        
        end% object.transform.RigidBody Constructor
                
        function coordMapped = mtimes(obj,coord)
        
            %% Overloaded * operator
            %
            Rotation    = obj.Rotation ;
            
            %
            Translation = obj.Translation ;
            
            %
            R           = coord.Rotation ;
            
            %
            T           = coord.Translation ;
            
            %
            coordMapped = object.transform.RigidBody(Rotation*R, Rotation*T + Translation) ;
                        
        end% Operator *
        
        function rotation = get.Rotation(obj)
            rotation = obj.R.R ;
        end
        
        function translation = get.Translation(obj)
            translation = obj.t ;
        end
        
        function disp(obj)
            obj.Rotation
            obj.Translation
        end
        
    end% methods
    
end
