classdef Rotation
    %object.transform.Rotation Provides rotation in Euler space.
    %
    % This transformation maps Euclidean 3D points to Euclidean 3D points,
    % preserving collineation, incidency, lengths, and angles.
    %
    %  @author  enrique.munoz@iit.it
    %
    %  @projectid  OMRON.P37505
    %
    %  @date    2013-06-05
    %
    %  @lastmod 2013-06-05
    %
    %  @changelog 2013-06-05  enrique.munoz@iit.it
    %
    %             Initial version
    %

    properties (Hidden=true)
        
        Data 
        
    end
    
    
    properties %(Dependent = true)
        %% Properties
        
        R      = eye(3) ;
        % 3x3 matrix defining the rotation.
        
        anglex = 0 ;
        % Rotation angle in the X-axis
        
        angley = 0 ;
        % Rotation angle in the Y-axis
        
        anglez = 0 ;
        % Rotation angle in the Z-axis
        
    end% Properties
    
    methods
        %% Methods
        
        function obj = Rotation(x, y, z)
        %% Constructor for RigidBody class    
        
        anglex = 0 ;
        angley = 0 ;
        anglez = 0 ;
        
        if (nargin > 0)
            if (nargin == 1)
                % Check if R is a rotation matrix
                if (~isequal(size(x), [3 3]))
                    error('Rotation must be a 3x3 matrix') ;
                else
                    if (isequal(size(x), [3 3]))
                        R      = x ;
                    else
                        R      = euler2rotmatrix(x, x, x) ;
                    end
                end
            end
            
            if (nargin == 2)
                error(['Wrong number of parameters: must be 0, 1, or 3']) ;
            end
            
            if (nargin == 3)
                % Check if R is a rotation matrix
                R      = euler2rotmatrix(x, y, z) ;
                anglex = x ;
                angley = y ;
                anglez = z ;
            end
            
            % Assign the rotation matrix
            obj.R      = R ;
            
            % Assign the Euler angles
            obj.anglex = anglex ;
            obj.angley = angley ;
            obj.anglez = anglez ;
        end
        end% object.transform.RotationConstructor

        
    end% methods
    
end
