classdef Frame 
    %sequence.Frame Represents the information of a single frame of the video
    %sequence: image data, ground truth information, sequence order, etc.
    %
    %  @author  enrique.munoz@iit.it
    %
    %  @projectid  OMRON.P37505
    %
    %  @date    2013-06-05
    %
    %  @lastmod 2013-06-05
    %
    %  @changelog 2013-06-05  enrique.munoz@iit.it
    %
    %             Initial version
    %
    %  @changelog 2014-04-10  enrique.munoz@iit.it
    %
    %             Completely refurbished the class to integrate it into
    %             top.Sequence.
    %

    properties
        %% Properties
        
        % Image raw data
        ImageData
        
        % Order in the sequence
        FrameNumber
        
        % Name of video image file
        ImageName 
        
        % Numner of motion parameters
        NumParameters

        % Name of motion parameters
        ParametersNames
        
        % Processing time for a given frame
        Time
        
        % Parameters estimated by the algorithms.
        EstimatedParameters
        
    end% Properties
    
    properties (Dependent)
        %% Properties
        
        % 
        Parameters
    
    end
    
    methods
        %% Methods
        
        %
        function obj = Frame(frame) ;
            %% Default constructor
                        
            % The Frame object is initialized with a frame number (order in
            % the sequence) 
            obj.FrameNumber = frame ;
            
        end % sequence.Frame constructor
    
        %
        function obj = assign(obj, params)
            % Assigns all the parameters of the Frame to the values in
            % params. 
        end% sequence.Frame.assign
                
        %
        function params = vec(obj)
            
            % Retrieve the frame parameters in vector form
            params = [] ;
        
        end% sequence.Frame.assign
                
        %
        function nam = names(obj)
            % Retrieves a cell array with the names of the parameters.
            nam = obj.ParametersNames ;
            
        end% sequence.Frame.assign
        
        %
        function ret = get.Parameters(obj)
           
            %
            ret = vec(obj) ;
            
        end
        
        %
        function obj = set.Parameters(obj, params)
           
            %
            obj.Parameters = params ;
            
        end
                
    end% methods
    
end