classdef FrameRigidBodyVICON < sequence.FrameRigidBodyPovray
    %sequence.FrameRigidBody Represents the information of a single frame of the video
    %sequence: image data, ground truth information, sequence order, etc.
    %
    %  The ground truth information stores rigid body transformations
    %  (rotation + translation) in R3.
    %
    %  @author  enrique.munoz@iit.it
    %
    %  @projectid  OMRON.P37505
    %
    %  @date    2014-06-15
    %
    %  @lastmod 2014-06-15
    %
    %  @changelog 2014-06-15  enrique.munoz@iit.it
    %
    %             Initial version
    %

    properties 
        %% Properties
        
        % 3D location of the captured VICON markers
        Trajectory
        
        % Rigid Body transformation between the first frame of the sequence
        % and the remaining frames
        Motion

        % Name of the file that will be generated with POVRAY to training
        % the linear regressor 
        TrainingImageName
        
        % Camera pose used to generate the training images
        TrainingViewPoint
        
    end% properties
    
    methods
        %% Methods
        
        %
        function obj = FrameRigidBodyVICON(frame) ;
            %% Default constructor
                        
            % The Frame object is initialized with a frame number (order in
            % the sequence)  
            obj = obj@sequence.FrameRigidBodyPovray(frame) ;
            
            % Six parameters as we are dealing with rigid body
            % transformations
            obj.NumParameters = 6 ;

            % The name of the parameters: 3 corresponding to the rotation
            % (Euler angles alpha, beta, and gamma)
            obj.ParametersNames = {'Alpha','Beta','Gamma','Tx','Ty','Tz'} ;

        end % sequence.Frame constructor
    
        %
        function obj = assign(obj, data)
            % Assigns all the parameters of the Frame to the values in
            % params. 
            obj.Alpha      = data.Rx ;
            obj.Beta       = data.Ry ;
            obj.Gamma      = data.Rz ;
            obj.Tx         = data.Tx ;
            obj.Ty         = data.Ty ;
            obj.Tz         = data.Tz ;
            
            %
            obj.Trajectory = data.Trajectory ;
            
            %
            obj.Motion     = data.Motion ;
            
        end% sequence.Frame.assign
                


       
    end% methods
    
end