%  @author  enrique.munoz@iit.it
%
%  @projectid  OMRON.P37505
%
%  @date    2013-06-01
%
%  @lastmod 2013-06-01
%
%  @changelog 2013-06-01  enrique.munoz@iit.it
%
%             Initial version
%

classdef DataLoaderVICON < handle
    %
    properties
        
        %
        ViewPoint
       
        %
        Frames

        %
        NumFrames 
        
        %
        GroundTruth 
        
        %
        Configuration
        
        %
        FrameFileExtension = 'png'
        
        %
        Scale              = 100 
        
        %
        % Default height for VICON images
        ImageHeight        = 964 
        
        % Default width for VICON images
        ImageWidth         = 1292 
        
    end
    
    %
    methods
        
        %
        function obj = DataLoaderVICON(sessionDir, captureName, ext, camSelected)
            
            %
            if (nargin > 2)
               
                obj.FrameFileExtension = ext ;
            
            end
            
            % Create basic scene configuration
            d                             = data_setup([]) ;
            obj.Configuration             = d.conf ;
            
            % 
            obj.Configuration.dir.session = [obj.Configuration.dir.sequence,filesep,...
                                             sessionDir] ;
            
            %
            obj.Configuration.dir.capture = [obj.Configuration.dir.session,filesep,captureName] ;       
            
            %
            obj.Configuration.dir.datafiles = [obj.Configuration.dir.capture,filesep,captureName] ;       
            
            %
            obj.Configuration.dir.frames    = [obj.Configuration.dir.capture,filesep,'frames'] ;       

            % Load the sequence filenames
            obj.loadSequence() ;
            
            % Load VICON capture data
            obj.loadDataGroundTruth() ;
            
            % Load VICON camera calibration parameters
            obj.loadDataConfiguration() ;
           
        end
                
        %% Methods
        function obj = loadDataGroundTruth(obj)
            
            dataFile = [obj.Configuration.dir.datafiles,'.csv'] ;
            
            % Open the data file
            fid = fopen(dataFile,'r'); %# open csv file for reading
            
            % Scan data file until get to the segments part
            bIdFound = false ;
            while ((~feof(fid)) & (~bIdFound))
                line     = fgets(fid); %# read line by line
                stringId = sscanf(line,'%s'); %# sscanf can read only numeric data :(
                
                % Stop when we reach the trajectories section
                if strcmp(stringId, 'Segments')
                    bIdFound = true ;
                end
            end
            
            % Exit if there is not trajectories section in the file
            if (~bIdFound)
                % Close file descriptor
                fclose(fid);
                
                error('This data file does not contain motion information') ;
            end
            
            % Jump into the data section
            numLines = 4 ;
            for count = 1:numLines
               line = fgets(fid) ; 
            end

            bSectionFinished = false ;
            % Read the data from the file
            while (~feof(fid) & (~bSectionFinished))
                
                line      = fgets(fid); %# read line by line
                
                % Format the line data
                data      = textscan(line,'%f','delimiter',',') ;
                data      = data{1} ;

                if (isempty(data))
                    bSectionFinished = true ;
                else
                    % Frame number
                    frame     = data(1) ;
                    
                    % Store the ground truth data
                    obj.GroundTruth(frame).Rx = data(3) ;
                    obj.GroundTruth(frame).Ry = data(4) ;
                    obj.GroundTruth(frame).Rz = data(5) ;
                    obj.GroundTruth(frame).Tx = data(6) ;
                    obj.GroundTruth(frame).Ty = data(7) ;
                    obj.GroundTruth(frame).Tz = data(8) ;
                end
            end
            
            % Scan data file until get to the trajectories part
            bIdFound = false ;
            while ((~feof(fid)) & (~bIdFound))
                line     = fgets(fid); %# read line by line
                stringId = sscanf(line,'%s'); %# sscanf can read only numeric data :(
                
                % Stop when we reach the trajectories section
                if strcmp(stringId, 'Trajectories')
                    bIdFound = true ;
                end
            end

            % Exit if there is not trajectories section in the file
            if (~bIdFound)
                % Close file descriptor
                fclose(fid);
                
                error('This data file does not contain trajectories information') ;
            end
            
            % Jump into the data section
            numLines = 4 ;
            for count = 1:numLines
               line = fgets(fid) ; 
            end
            
            lenData = 0 ;
            
            bSectionFinished = false ;
            % Read the data from the file
            while (~feof(fid) & (~bSectionFinished))
                
                line      = fgets(fid); %# read line by line
                
                % Format the line data
                data      = textscan(line,'%f','delimiter',',') ;
                data      = data{1} ;

                % We assume the first line does not have empty values
                if (lenData == 0)
                    lenData = numel(data) ;
                end
                    
                if (isempty(data))
                    bSectionFinished = true ;
                else
                    
                    % Detect empty values at the end of the array
                    if (numel(data)  < lenData)
                        
                        % we augment the array with NaN
                        data = [data; NaN] ;
                                       
                    end
                    
                    % Frame number
                    frame     = data(1) ;
                    
                    % Number of data points
                    numPoints = (length(data) - 2)/3 ;
                    
                    % 3D points trajectories at the current frame
                    points    = reshape(data(3:end),3,numPoints) ;
                    
                    % Store the ground truth data
                    obj.GroundTruth(frame).Trajectory = points ;
                end
            end
            
            % The ration vicon:basler is 2:1
            numFrames                      = length(obj.GroundTruth) ;
            obj.GroundTruth(2:2:numFrames) = [] ;
            
            % Close file descriptor
            fclose(fid);
            
            obj.computeGroundTruthMotion() ;
       
            %
            obj.NumFrames = obj.NumFrames - 1 ;

        end
        
        %
        function obj = computeGroundTruthMotion(obj)
        
            % We set the initial motion to the identity
            obj.GroundTruth(1).Motion = object.transform.RigidBody(eye(3),[0;0;0]) ;
            
            % Reference structure
            refPoints = obj.GroundTruth(1).Trajectory ;

            for frame = 2:(obj.NumFrames-1)
                
                % Compute the relative motion between the current and the
                % previous frames
                points  = obj.GroundTruth(frame).Trajectory ;
                
                % Only pick those values different from NaN
                
                indexesNotEmpty = find(prod(~isnan(points))) ;
                
                if (~isempty(indexesNotEmpty)) & (~isempty(points))
                    
                    numPoints = numel(indexesNotEmpty) ;
                    
                    % Compute the motion between frames using Procrustes
                    [dist,transPoints,transform] = ...
                        procrustes(points(:,indexesNotEmpty)',...
                        refPoints(:,indexesNotEmpty)',...
                        'Scaling', false,'Reflection',false) ;
                    
                    % Extract motion parameters
                    %                 Rcurrent = (transform.b*transform.T)' ;
                    Rcurrent = (transform.T)' ;
                    tcurrent = transform.c ;
                    tcurrent = tcurrent(1,:)' ;
                    
                    points(:,indexesNotEmpty) - (Rcurrent*refPoints(:,indexesNotEmpty)+repmat(tcurrent,1,numPoints)) ;
                    
                    % Store data
                    obj.GroundTruth(frame).Motion = object.transform.RigidBody(Rcurrent,tcurrent) ;
                end
            end
            
        end
        
        %
        function obj = loadSequence(obj, folderName)
            
            %
            d2             = dir([obj.Configuration.dir.frames,'/*.', obj.FrameFileExtension]) ;
            obj.NumFrames = length(d2) ;
             
            %
            for frame = 1:obj.NumFrames
                
                %
                obj.Frames(frame).Camera2.Name = d2(frame).name(1:end-4) ;

            end
            
        end
        
        %
        function obj = loadDataConfiguration(obj)
    
            %
            xmlfile = [obj.Configuration.dir.datafiles, '.xcp'] ;
            xmldata = xml2struct(xmlfile) ;
                
            % WARNING!!!! These values are tailored for a given VICON
            % camera configuration. If the configuration of the cameras
            % changes, these values must be accordingly modified
            indcam = 14 ;
            
            %
            cam.focal_length = str2num(xmldata.Children(indcam).Children(6).Children(2).Attributes(1).Value) ;
            cam.orientation  = str2num(xmldata.Children(indcam).Children(6).Children(2).Attributes(4).Value) ;
            cam.position     = str2num(xmldata.Children(indcam).Children(6).Children(2).Attributes(5).Value) ;
            cam.ppoint       = str2num(xmldata.Children(indcam).Children(6).Children(2).Attributes(6).Value) ;
           
            %
            K   = [cam.focal_length  0     cam.ppoint(1);...
                    0     cam.focal_length  cam.ppoint(2);...
                    0      0     1] ;                     
            
            %
            R   = q2dcm(cam.orientation) ;
                
            %
            t   = cam.position' ;   

            %
            obj.ViewPoint.K = K ;
            obj.ViewPoint.R = R' ;
            obj.ViewPoint.t = -R'*t;
            
            % Set the default image dimensions to the viewport
            obj.ViewPoint.Height = obj.ImageHeight ;
            obj.ViewPoint.Width  = obj.ImageWidth ;

        end% 
        
    end% methods
    
end
