%  @author  enrique.munoz@iit.it
%
%  @projectid  OMRON.P37505
%
%  @date    2013-06-01
%
%  @lastmod 2013-06-01
%
%  @changelog 2013-06-01  enrique.munoz@iit.it
%
%             Initial version
%
classdef DataLoaderVICONEdges < vicon.DataLoaderVICON

    properties
        
        % Rigid body transformation between the camera and the origin of
        % the object coordinate system
        CalibrationModel
        
        % 
        Model

        %
        Noise
        
        %
        SamplesPerNoise
        
    end
    
    %
    methods
     
        %
        function obj = DataLoaderVICONEdges(varargin)
            
            %
            obj = obj@vicon.DataLoaderVICON(varargin{:})
           
        end

        %
        function alignCoordinateSystem(obj, refPoints, origin, model)
        
            % Origin alignment motion
            Ralign = origin.Rotation ;
            talign = origin.Translation ;
            
            %
            Malign = object.transform.RigidBody(Ralign, talign) ;
            
            % Find the transformation between the marker points (the
            % trajectories) captured by VICON and the reference system on
            % the calibrated grid
            points = obj.GroundTruth(1).Trajectory ;
            
            [~,~,transform]=procrustes(points',...
                                       refPoints',...
                                       'Scaling',false) ;
                                                
            % Extract motion parameters
            Rcurrent = (transform.T)' ;
            tcurrent = transform.c ;
            tcurrent = tcurrent(1,:)' ;
                
            % Store data
            obj.CalibrationModel.Motion = object.transform.RigidBody(Rcurrent,tcurrent) ;
        
            % Align the ground truth data to the object reference system
            for frame = 1:obj.NumFrames
                
                % 
                R = obj.GroundTruth(frame).Motion.Rotation ;
                
                % 
                t = obj.GroundTruth(frame).Motion.Translation ;
                
                %
                Rmotion = R*Rcurrent ;
               
                %
                tmotion = R*tcurrent + t ;
                
                %
                obj.GroundTruth(frame).Motion = object.transform.RigidBody(Rmotion*Ralign,Rmotion*talign + tmotion) ; 
                
            end
            
%             % Set the origin as the center of the object reference system
            obj.Model   = model ;
            
        end
        
    end% methods
    
end
